@@ -1,15 +1,15 @@
 /* Generate a file containing some preset patterns.
    Print statistics for existing files.
 
-   Copyright (C) 1995, 1996, 1997, 2001, 2003, 2004, 2005 Free Software
-   Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 2001, 2003, 2004, 2005, 2006, 2007,
+   2008 Free Software Foundation, Inc.
 
    François Pinard <pinard@iro.umontreal.ca>, 1995.
-   Sergey Poznyakoff <gray@mirddin.farlep.net>, 2004, 2005.
+   Sergey Poznyakoff <gray@mirddin.farlep.net>, 2004, 2005, 2006, 2007, 2008.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful, but
@@ -29,9 +29,13 @@
 #include <argp.h>
 #include <argcv.h>
 #include <getdate.h>
-#include <setenv.h>
 #include <utimens.h>
 #include <inttostr.h>
+#include <fcntl.h>
+#include <sys/stat.h>
+#define obstack_chunk_alloc malloc
+#define obstack_chunk_free free
+#include <obstack.h>
 
 #ifndef EXIT_SUCCESS
 # define EXIT_SUCCESS 0
@@ -56,8 +60,13 @@ const char *program_name;
 /* Name of file to generate */
 static char *file_name;
 
+/* Name of the file-list file: */
+static char *files_from;
+static char filename_terminator = '\n';
+
 /* Length of file to generate.  */
 static off_t file_length = 0;
+static off_t seek_offset = 0;
 
 /* Pattern to generate.  */
 static enum pattern pattern = DEFAULT_PATTERN;
@@ -109,6 +118,7 @@ static char doc[] = N_("genfile manipulates data files for GNU paxutils test sui
 #define OPT_EXEC       260
 #define OPT_DATE       261
 #define OPT_VERBOSE    262
+#define OPT_SEEK       263
 
 static struct argp_option options[] = {
 #define GRP 0
@@ -118,6 +128,10 @@ static struct argp_option options[] = {
    N_("Create file of the given SIZE"), GRP+1 },
   {"file", 'f', N_("NAME"), 0,
    N_("Write to file NAME, instead of standard output"), GRP+1},
+  {"files-from", 'T', N_("FILE"), 0,
+   N_("Read file names from FILE"), GRP+1},
+  {"null", '0', NULL, 0,
+   N_("-T reads null-terminated names"), GRP+1},
   {"pattern", 'p', N_("PATTERN"), 0,
    N_("Fill the file with the given PATTERN. PATTERN is 'default' or 'zeros'"),
    GRP+1 },
@@ -126,6 +140,10 @@ static struct argp_option options[] = {
   {"sparse", 's', NULL, 0,
    N_("Generate sparse file. Rest of the command line gives the file map."),
    GRP+1 },
+  {"seek", OPT_SEEK, N_("OFFSET"), 0,
+   N_("Seek to the given offset before writing data"),
+   GRP+1 },
+
 #undef GRP
 #define GRP 10
   {NULL, 0, NULL, 0,
@@ -135,6 +153,7 @@ static struct argp_option options[] = {
    N_("Print contents of struct stat for each given file. Default FORMAT is: ")
    DEFAULT_STAT_FORMAT,
    GRP+1 },
+
 #undef GRP
 #define GRP 20
   {NULL, 0, NULL, 0,
@@ -178,8 +197,10 @@ static char const * const pattern_args[] = { "default", "zeros", 0 };
 static enum pattern const pattern_types[] = {DEFAULT_PATTERN, ZEROS_PATTERN};
 
 static int
-xlat_suffix (off_t *vp, char *p)
+xlat_suffix (off_t *vp, const char *p)
 {
+  off_t val = *vp;
+
   if (p[1])
     return 1;
   switch (p[0])
@@ -200,19 +221,52 @@ xlat_suffix (off_t *vp, char *p)
     default:
       return 1;
     }
-  return 0;
+  return *vp <= val;
 }
 
 static off_t
 get_size (const char *str, int allow_zero)
 {
-  char *p;
-  off_t n;
+  const char *p;
+  off_t v = 0;
 
-  n = strtoul (str, &p, 0);
-  if (n < 0 || (!allow_zero && n == 0) || (*p && xlat_suffix (&n, p)))
-    error (EXIT_FAILURE, 0, _("Invalid size: %s"), str);
-  return n;
+  for (p = str; *p; p++)
+    {
+      int digit = *p - '0';
+      off_t x = v * 10;
+      if (9 < (unsigned) digit)
+	{
+	  if (xlat_suffix (&v, p))
+	    error (EXIT_FAILURE, 0, _("Invalid size: %s"), str);
+	  else
+	    break;
+	}
+      else if (x / 10 != v)
+	error (EXIT_FAILURE, 0, _("Number out of allowed range: %s"), str);
+      v = x + digit;
+      if (v < 0)
+	error (EXIT_FAILURE, 0, _("Negative size: %s"), str);
+    }
+  return v;
+}
+
+void
+verify_file (char *file_name)
+{
+  if (file_name)
+    {
+      struct stat st;
+
+      if (stat (file_name, &st))
+	error (0, errno, _("stat(%s) failed"), file_name);
+
+      if (st.st_size != file_length + seek_offset)
+	error (1, 0, _("requested file length %lu, actual %lu"),
+	       (unsigned long)st.st_size, (unsigned long)file_length);
+
+      if (mode == mode_sparse && !ST_IS_SPARSE (st))
+	error (1, 0, _("created file is not sparse"));
+    }
 }
 
 struct action
@@ -228,7 +282,7 @@ struct action
 
 static struct action *action_list;
 
-int
+void
 reg_action (int action, char *arg)
 {
   struct action *act = xmalloc (sizeof (*act));
@@ -247,6 +301,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
 {
   switch (key)
     {
+    case '0':
+      filename_terminator = 0;
+      break;
+
     case 'f':
       file_name = arg;
       break;
@@ -278,6 +336,14 @@ parse_opt (int key, char *arg, struct argp_state *state)
       argcv_get (arg, "", NULL, &exec_argc, &exec_argv);
       break;
 
+    case 'T':
+      files_from = arg;
+      break;
+
+    case OPT_SEEK:
+      seek_offset = get_size (arg, 0);
+      break;
+
     case OPT_CHECKPOINT:
       {
 	char *p;
@@ -342,28 +408,68 @@ fill (FILE *fp, off_t length, enum pattern pattern)
 
 /* Generate Mode: usual files */
 static void
-generate_simple_file (int argc, char **argv)
+generate_simple_file (char *filename)
 {
-  int i;
   FILE *fp;
 
-  if (argc)
-    error (EXIT_FAILURE, 0, _("too many arguments"));
-
-  if (file_name)
+  if (filename)
     {
-      fp = fopen (file_name, "w");
+      fp = fopen (filename, seek_offset ? "rb+" : "wb");
       if (!fp)
-	error (EXIT_FAILURE, 0, _("cannot open `%s'"), file_name);
+	error (EXIT_FAILURE, errno, _("cannot open `%s'"), filename);
     }
   else
     fp = stdout;
 
+  if (fseeko (fp, seek_offset, 0))
+    error (EXIT_FAILURE, errno, "%s", _("cannot seek"));
+
   fill (fp, file_length, pattern);
 
   fclose (fp);
 }
 
+/* A simplified version of the same function from tar */
+int
+read_name_from_file (FILE *fp, struct obstack *stk)
+{
+  int c;
+  size_t counter = 0;
+
+  for (c = getc (fp); c != EOF && c != filename_terminator; c = getc (fp))
+    {
+      if (c == 0)
+	error (EXIT_FAILURE, 0, _("file name contains null character"));
+      obstack_1grow (stk, c);
+      counter++;
+    }
+
+  obstack_1grow (stk, 0);
+
+  return (counter == 0 && c == EOF);
+}
+
+void
+generate_files_from_list ()
+{
+  FILE *fp = strcmp (files_from, "-") ? fopen (files_from, "rb") : stdin;
+  struct obstack stk;
+
+  if (!fp)
+    error (EXIT_FAILURE, errno, _("cannot open `%s'"), files_from);
+
+  obstack_init (&stk);
+  while (!read_name_from_file (fp, &stk))
+    {
+      char *name = obstack_finish (&stk);
+      generate_simple_file (name);
+      verify_file (name);
+      obstack_free (&stk, name);
+    }
+  fclose (fp);
+  obstack_free (&stk, NULL);
+}
+
 
 /* Generate Mode: sparse files */
 
@@ -378,14 +484,14 @@ mkhole (int fd, off_t displ)
 static void
 mksparse (int fd, off_t displ, char *marks)
 {
+  if (lseek (fd, displ, SEEK_CUR) == -1)
+    error (EXIT_FAILURE, errno, "lseek");
+
   for (; *marks; marks++)
     {
       memset (buffer, *marks, block_size);
       if (write (fd, buffer, block_size) != block_size)
 	error (EXIT_FAILURE, errno, "write");
-
-      if (lseek (fd, displ, SEEK_CUR) == -1)
-	error (EXIT_FAILURE, errno, "lseek");
     }
 }
 
@@ -394,19 +500,25 @@ generate_sparse_file (int argc, char **argv)
 {
   int i;
   int fd;
+  int flags = O_CREAT | O_RDWR | O_BINARY;
 
   if (!file_name)
     error (EXIT_FAILURE, 0,
 	   _("cannot generate sparse files on standard output, use --file option"));
-  fd = open (file_name, O_CREAT|O_TRUNC|O_RDWR, 0644);
+  if (!seek_offset)
+    flags |= O_TRUNC;
+  fd = open (file_name, flags, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
   if (fd < 0)
-    error (EXIT_FAILURE, 0, _("cannot open `%s'"), file_name);
+    error (EXIT_FAILURE, errno, _("cannot open `%s'"), file_name);
 
   buffer = xmalloc (block_size);
 
+  file_length = 0;
+
   for (i = 0; i < argc; i += 2)
     {
       off_t displ = get_size (argv[i], 1);
+      file_length += displ;
 
       if (i == argc-1)
 	{
@@ -414,7 +526,10 @@ generate_sparse_file (int argc, char **argv)
 	  break;
 	}
       else
-	mksparse (fd, displ, argv[i+1]);
+	{
+	  file_length += block_size * strlen (argv[i+1]);
+	  mksparse (fd, displ, argv[i+1]);
+	}
     }
 
   close (fd);
@@ -445,7 +560,7 @@ print_stat (const char *name)
     }
 
   fmt = strdup (stat_format);
-  for (p = strtok (fmt, ","); p; p = strtok (NULL, ","))
+  for (p = strtok (fmt, ","); p; )
     {
       if (memcmp (p, "st_", 3) == 0)
 	p += 3;
@@ -455,8 +570,28 @@ print_stat (const char *name)
 	printf ("%lu", (unsigned long) st.st_dev);
       else if (strcmp (p, "ino") == 0)
 	printf ("%lu", (unsigned long) st.st_ino);
-      else if (strcmp (p, "mode") == 0)
-	printf ("%0o", st.st_mode);
+      else if (strncmp (p, "mode", 4) == 0)
+	{
+	  mode_t mask = ~0;
+
+	  if (ispunct (p[4]))
+	    {
+	      char *q;
+
+	      mask = strtoul (p + 5, &q, 8);
+	      if (*q)
+		{
+		  printf ("\n");
+		  error (EXIT_FAILURE, 0, _("incorrect mask (near `%s')"), q);
+		}
+	    }
+	  else if (p[4])
+	    {
+	      printf ("\n");
+	      error (EXIT_FAILURE, 0, _("Unknown field `%s'"), p);
+	    }
+	  printf ("%0o", st.st_mode & mask);
+	}
       else if (strcmp (p, "nlink") == 0)
 	printf ("%lu", (unsigned long) st.st_nlink);
       else if (strcmp (p, "uid") == 0)
@@ -481,12 +616,16 @@ print_stat (const char *name)
 	printf ("%lu", (unsigned long) st.st_ctime);
       else if (strcmp (p, "ctimeH") == 0)
 	print_time (st.st_ctime);
+      else if (strcmp (p, "sparse") == 0)
+	printf ("%d", ST_IS_SPARSE (st));
       else
 	{
 	  printf ("\n");
 	  error (EXIT_FAILURE, 0, _("Unknown field `%s'"), p);
 	}
-      printf (" ");
+      p = strtok (NULL, ",");
+      if (p)
+	printf (" ");
     }
   printf ("\n");
   free (fmt);
@@ -517,7 +656,7 @@ exec_checkpoint (struct action *p)
 
     case OPT_APPEND:
       {
-	FILE *fp = fopen (p->name, "a");
+	FILE *fp = fopen (p->name, "ab");
 	if (!fp)
 	  {
 	    error (0, errno, _("cannot open `%s'"), p->name);
@@ -531,7 +670,7 @@ exec_checkpoint (struct action *p)
 
     case OPT_TRUNCATE:
       {
-	int fd = open (p->name, O_RDWR);
+	int fd = open (p->name, O_RDWR | O_BINARY);
 	if (fd == -1)
 	  {
 	    error (0, errno, _("cannot open `%s'"), p->name);
@@ -582,7 +721,7 @@ process_checkpoint (size_t n)
 void
 exec_command (void)
 {
-  int i, status;
+  int status;
   pid_t pid;
   int fd[2];
   char *p;
@@ -593,8 +732,8 @@ exec_command (void)
      FIXME: This assumes that exec_argv does not use traditional tar options
      (without dash) */
   exec_argc++;
-  exec_argv = xrealloc (exec_argv, exec_argc * sizeof (*exec_argv));
-  memmove (exec_argv+2, exec_argv+1, (exec_argc-1)*sizeof (*exec_argv));
+  exec_argv = xrealloc (exec_argv, (exec_argc + 1) * sizeof (*exec_argv));
+  memmove (exec_argv+2, exec_argv+1, (exec_argc - 1) * sizeof (*exec_argv));
   exec_argv[1] = "--checkpoint";
 
 #ifdef SIGCHLD
@@ -626,7 +765,7 @@ exec_command (void)
 
   /* Master */
   close (fd[1]);
-  fp = fdopen (fd[0], "r");
+  fp = fdopen (fd[0], "rb");
   if (fp == NULL)
     error (EXIT_FAILURE, errno, "fdopen");
 
@@ -692,6 +831,9 @@ main (int argc, char **argv)
 
   program_name = argv[0];
   setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   get_date (&touch_time, "now", NULL);
 
   /* Decode command options.  */
@@ -714,10 +856,19 @@ main (int argc, char **argv)
 
     case mode_sparse:
       generate_sparse_file (argc, argv);
+      verify_file (file_name);
       break;
 
     case mode_generate:
-      generate_simple_file (argc, argv);
+      if (argc)
+	error (EXIT_FAILURE, 0, _("too many arguments"));
+      if (files_from)
+	generate_files_from_list ();
+      else
+	{
+	  generate_simple_file (file_name);
+	  verify_file (file_name);
+	}
       break;
 
     case mode_exec:
